/*
 *  SNAPMASS:  set masses in a snapshot
 *
 *	1-Aug-88	V1.0 created      				PJT
 *	5-dec-88	V1.1 allow all masses same through one keyword  PJT
 *				(proposed - to be coded )
 *	5-feb-89	V1.2 allow bodytrans(3NEMO) expression for mass PJT
 *	13-may-91	V1.3 added helpvec etc.				PJT
 *      23-nov-93       V1.3a cosmetic
 *      21-dec-95       V1.4 process all snapshots
 */
#include <stdinc.h>
#include <getparam.h>
#include <vectmath.h>
#include <filestruct.h>

#include <snapshot/snapshot.h>
#include <snapshot/body.h>
#include <snapshot/get_snap.c>
#include <snapshot/put_snap.c>

string defv[] = {		/* DEFAULT INPUT PARAMETERS */
    "in=???\n			input (snapshot) file",
    "out=???\n                  output (snapshot) file",
    "mass=1\n			expression for new masses",
    "inmass=\n   		extra file, if to use their masses only",
    "VERSION=1.3\n		13-may-91 PJT",
    NULL,
};

string usage="(re)assign masses to a snapshot";

#define TIMEFUZZ	0.0001	/* tolerance in time comparisons */


nemo_main()
{
    stream instr, inmassstr, outstr;
    char  *massstr;
    real   tsnap, tsnapmass, mass_star;
    Body  *btab = NULL, *bp;
    Body  *bmasstab = NULL, *bmassp;
    int i, nbody, nbodymass, bits, bitsmass;
    rproc  mfunc, btrtrans();
    bool  first = TRUE;

    instr = stropen(getparam("in"), "r");
    massstr = getparam("inmass");
    if (*massstr == NULL) {
        mfunc = btrtrans(getparam("mass"));     /* use bodytrans expression */
        inmassstr = NULL;
    } else
        inmassstr = stropen(massstr,"r");
    outstr = stropen(getparam("out"), "w");

    for(;;) {
					/* input main data */
    	get_history(instr);                    /* skip history & comments */
        if (!get_tag_ok(instr, SnapShotTag)) {
            break;
            error("Snapmass (in): Need a snapshot");
        }
        get_snap(instr, &btab, &nbody, &tsnap, &bits);
        if (bits&MassBit)
            dprintf(0,"Warning: existing masses overwritten\n");
        if (!(bits&TimeBit)) {
            dprintf(1,"Warning: time=0.0 assumed\n");
            tsnap = 0.0;
        }
        
        if (inmassstr!=NULL) { 		/* input mass data */
    	    get_history(inmassstr);
            if (!get_tag_ok(inmassstr, SnapShotTag))
       		error("Snapmass (inmass): Need a snapshot");
            get_snap(inmassstr, &bmasstab, &nbodymass, &tsnapmass, &bitsmass);
            if (nbodymass != nbody) {
                printf ("Warning: number of bodies in both snapshots unequal\n");
                printf ("Only mass of first body copied\n");
            }
            if ((bitsmass & MassBit) == 0) {
                unlink(getparam("out"));
                error("%s: essential mass data missing\tbits = %x\n", 
                            getargv0(), bits);
            }
        }
        				/* output */
        if (first) {
            put_history(outstr);
            first = FALSE;
        }
        if (inmassstr) {
	    if (nbody==nbodymass)
	        for (bp=btab, bmassp=bmasstab; bp<btab+nbody; bp++, bmassp++)
	            Mass(bp) = Mass(bmassp);
	    else
	        for (bp=btab, bmassp=bmasstab; bp<btab+nbody; bp++)
	            Mass(bp) = Mass(bmassp);
        } else
            for (bp=btab, i=0; i<nbody; bp++,i++)
                Mass(bp) = mfunc(bp, tsnap, i);

	bits |= MassBit;    /* turn mass bit on anyhow */
        put_snap(outstr, &btab, &nbody, &tsnap, &bits);
    }

    strclose(instr);
    if (inmassstr) strclose(inmassstr);
    strclose(outstr);
}


