/* list.c -	List manipulation routines
 *
 * SYNOPSIS
 *	void list_init (struct list *list) ;
 *
 *		Initializes the list.  This is actually only a macro,
 *		and it does not free any storage that may have been
 *		allocated to the list.
 *
 *	void list_free (struct list *list) ;
 *
 *		Deallocates and initializes the list.
 *
 *	void list_push (struct list *list, void *element) ;
 *
 *		Adds the element to the end of the list.
 *
 *	void *list_pop (struct list *list) ;
 *
 *		Returns the last element in the list and removes it.
 *		The list is automatically deallocated using list_free
 *		when the last element has been popped.
 *
 *	void *list_shift (struct list *list) ;
 *
 *		Returns the first element in the list and removes it
 *		The list is automatically deallocated using list_free
 *		when the last element has been shifted.
 *
 *	void list_sort (struct list *list, int (*compare) ()) ;
 *
 *		Sorts the list using compare() (like qsort(3))
 *
 * Copyright (c) 1991 Tim Cook.
 * Non-profit distribution allowed.  See README for details.
 */

static char rcsid[] = "$Header: list.c 1.0 91/03/22 $" ;

#define _LIST_C

#include <list.h>

#define NULL	0

extern void *malloc () ;
extern void *realloc () ;
extern int free () ;

#define INITIAL_ENTRIES	512


void list_free (list)
   struct list *list ;
{
   free (list->s_start) ;
   list->s_start = (void **) NULL ;
   }


void list_push (list, element)
   struct list *list ;
   void *element ;
{
   register unsigned int x, y, z ;

   if (list->start == (void **) NULL) {
      x = sizeof (void *) * INITIAL_ENTRIES ;
      list->s_start = (void **) malloc (x) ;
      list->s_end = list->s_start + INITIAL_ENTRIES ;
      list->start = list->s_start ;
      list->end = list->start ;
      *list->end = element ; }
   else {
      if (list->end >= list->s_end) {
	 x = (unsigned int) list->s_end - (unsigned int) list->s_start ;
	 y = (unsigned int) list->start - (unsigned int) list->s_start ;
	 z = (unsigned int) list->end - (unsigned int) list->s_start ;
	 x = x << 1 ;
	 list->s_start = (void **) realloc (list->s_start, x) ;
	 list->s_end = (void **) ((unsigned int) list->s_start + x) ;
	 list->start = (void **) ((unsigned int) list->s_start + y) ;
	 list->end = (void **) ((unsigned int) list->s_start + z) ; }
      list->end++ ;
      *list->end = element ; } }


void *list_shift (list)
   struct list *list ;
{
   if (list->start > list->end) {
      list_free (list) ;
      return (void *) NULL ; }
   else
      return *list->start++ ;
   }


void *list_pop (list)
   struct list *list ;
{
   if (list->start == (void **) NULL)
      return (void *) NULL ;
   else {
      if (list->end == list->start) {
	 list_free (list) ;
	 return *list->end ; }
      else
	return *list->end-- ; }
   }


void list_sort (list, compare)
   struct list *list ;
   int (*compare) () ;
{
   register unsigned int x ;

   x = (unsigned int) list->end - (unsigned int) list->start ;
   x = x / sizeof (void *)  + 1 ;
   qsort (list->start, x, sizeof (void *), compare) ;
   }
